/**
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#ifndef WLPDSTM_ALLOCATORS_H_
#define WLPDSTM_ALLOCATORS_H_

#include "memory.h"

namespace wlpdstm {

	/**
	 * Base class for all classes that are allocated using wlpdstm
	 * memory manager.
	 */
	class Alloced {
		public:
			virtual ~Alloced() { }
	};

	/**
	 * Marker class used when cloning objects.
	 */
	class Cloning {	};

	/**
	 * Transactionally allocated classes.
	 * 
	 * NOTE: not used in fully word-oriented apps.
	 */
	/*class WlpdstmTxAlloced : public Alloced {
		public:
			void *operator new(size_t size) {
				return CurrentTransaction::TxMalloc(size);
			}

			void *operator new(size_t size, Cloning cloning) {
				return CurrentTransaction::TxMalloc(size);
			}

			// don't do tx_free, as that will lead to infinite looping
			void operator delete(void* ptr) {
				MemoryManager::Free(ptr);
			}

			virtual ~WlpdstmTxAlloced() { }
	};*/

	/**
	 * Use non-tx free/malloc provided by wlpdstm.
	 */
	class WlpdstmAlloced : public Alloced {
		public:
			void* operator new(size_t size) {
				//return MemoryManager::Malloc(size);
				return malloc(size);
			}

			void operator delete(void* ptr) {
				//MemoryManager::Free(ptr);
				free(ptr);
			}

			virtual ~WlpdstmAlloced() { }
	};
}

#endif // WLPDSTM_ALLOCATORS_H_
