/**
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#include <cassert>

#include "memory_explicit_fast.h"
#include "../common/atomic.h"
#include "../constants.h"

// backing storage for timestamp
volatile wlpdstm::v_timestamp wlpdstm::global_timestamp
		__attribute__ ((aligned(64)));

//static wlpdstm::thread_egc_data global_egc[MAX_THREADS];

// backing storage for tls gc_key
//pthread_key_t wlpdstm::gc_key;

void wlpdstm::global_init_mem() {
	Tls<thread_egc_data>::GlobalInit();
	// for the time being don't worry about freeing this data
	//::pthread_key_create(&gc_key, NULL);
}

void wlpdstm::init_thread_mem() {
	// get thread identifier
	unsigned tid = wlpdstm::get_tid();

	// initialize tls data
	//wlpdstm::thread_egc_data *egc = (thread_egc_data *)wlpdstm::malloc(
	//	sizeof(thread_egc_data));
	Tls<thread_egc_data>::ThreadInit();
	wlpdstm::thread_egc_data *egc = Tls<thread_egc_data>::Get();

	//wlpdstm::thread_egc_data *egc = global_egc + tid;

	// initialize fields
	egc->freed = NULL;

	egc->free_on_abort = NULL;

	egc->free_on_commit = NULL;

	egc->free_on_commit_dominated = NULL;

	egc->pool_free_chunk = alloc_pool_free_chunk();
	egc->free_chunk_idx = 0;
	egc->pool_free_list = NULL;

	//pthread_setspecific(gc_key, egc);

	assert(tid < MAX_THREADS);

	// update global timestamp length
	unsigned current_ts_len;

	while(true) {
		current_ts_len = atomic_load_no_barrier(&global_timestamp.len);

		if(current_ts_len < tid + 1) {
			if(atomic_cas_no_barrier(&global_timestamp.len, current_ts_len, tid + 1)) {
				break;
			}
		} else {
			break;
		}
	}
}
