/**
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#ifndef WLPDSTM_TRANSACTION_H_
#define WLPDSTM_TRANSACTION_H_

#include "memory.h"
#include "common/tls.h"
#include "aborted_exc.h"

///////////////////////////////////////////
// different transaction implementations //
///////////////////////////////////////////

#include "transaction_mixinv_ir.h"

///////////////////////////////////////
// select transaction implementation //
///////////////////////////////////////

namespace wlpdstm {

	typedef TxMixinvIR Transaction;
}

/////////////////////////////////////
// thread local transaction object //
/////////////////////////////////////

namespace wlpdstm {

	// A shortcut used for accessing transaction in progress.
	class CurrentTransaction : public Tls<Transaction, true, true> {
		public:
			static void TxStart() {
				Tls<Transaction, true, true>::Get()->TxStart();
			}

			static void TxCommit() {
				Tls<Transaction, true, true>::Get()->TxCommit();
			}

			static void TxRestart() {
				Tls<Transaction, true, true>::Get()->TxRestart();
			}

			static void TxAbort() {
				Tls<Transaction, true, true>::Get()->TxAbort();
			}

			static void WriteWord(void *address, void *value) {
				Tls<Transaction, true, true>::Get()->WriteWord(address, value);
			}

			static void WriteWordLocal(void *address, void *value) {
				Tls<Transaction, true, true>::Get()->WriteWordLocal(address, value);
			}

			static void *ReadWord(void *address) {
				return Tls<Transaction, true, true>::Get()->ReadWord(address);
			}

			static void *TxMalloc(size_t size) {
				return Tls<Transaction, true, true>::Get()->TxMalloc(size);
			}

			static void TxFree(void *address, size_t size) {
				return Tls<Transaction, true, true>::Get()->TxFree(address, size);
			}
	};
}

#endif // WLPDSTM_TRANSACTION_H_
