/**
 * @author Aleksandar Dragojevic aleksandar.dragojevic@epfl.ch
 */

#ifndef WLPDSTM_WLPDSTM_H_
#define WLPDSTM_WLPDSTM_H_

#include "aborted_exc.h"
#include "tid.h"
#include "memory.h"
#include "transaction.h"
#include "constants.h"

namespace wlpdstm {

	// initialization
	void global_init();

	void thread_init();

	// start/end tx
	void start_tx();

	void commit_tx();

	void abort_tx();

	void restart_tx();

	// read/write word
	void *read_word(void *address);

	void write_word(void *address, void *value);

	void write_word_local(void *address, void *value);

	// memory management
	void *tx_malloc(size_t size);

	void tx_free(void *ptr, size_t size);

	///////////////////////////////////////////////////////
	// a separate set of methods if current tx is cached //
	///////////////////////////////////////////////////////

	// start/end tx
	void start_tx(Transaction *tx);

	void commit_tx(Transaction *tx);

	void abort_tx(Transaction *tx);

	void restart_tx(Transaction *tx);

	// read/write word
	void *read_word(Transaction *tx, void *address);

	void write_word(Transaction *tx, void *address, void *value);

	void write_word_local(Transaction *tx, void *address, void *value);

	// memory management
	void *tx_malloc(Transaction *tx, size_t size);

	void tx_free(Transaction *tx, void *ptr, size_t size);
}

#endif // WLPDSTM_WLPDSTM_H_
